#!/usr/bin/python
#
from __future__ import division
import time
import math
import json
import time
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication

version = 1.0


def acquire_histograms(dwell_time, num_histos, file_name):
    """
    We assume that only one MCA-1000 is attached, and that the MCA Data Server is running.

    :param dwell_time: Acquisition time in seconds
    :param num_histos: Number of histograms to acquire and write to file
    :param file_name: Write the histogram to this file as a on-line json file
    :return: None
    """
    mds_client, sn = bpi_communication.connect_one_device()

    start_mca_cmd = {"type": "mca_cmd", "name": "arm_ctrl",  "dir": "rmw", "sn": sn,
               "data": {"fields": {"run_time_sample": dwell_time},
               "user":{"histogram_run": 1, "active_bank": 0, "clear_statistics": 1, "clear_histogram": 1}}}
    read_histogram_cmd = {"type": "mca_cmd", "name": "arm_histogram", "dir": "read", "sn": sn}
    read_status_cmd = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}

    # Delete a pre-existing file
    with open(file_name, 'w') as fout:
        pass

    for n in range(num_histos):
        # Start new sample histogram acquisition
        ret = mds_client.sar(start_mca_cmd)

        time.sleep(dwell_time+1)  # wait

        # Read count rate and histogram data
        ret = mds_client.sar(read_status_cmd)
        status_data = ret[sn]["fields"]

        ret = mds_client.sar(read_histogram_cmd)
        mca_data = ret[sn]["fields"]

        out_dict = {"histogram": mca_data, "status": status_data}
        with open(file_name, 'a') as fout:
            fout.write(json.dumps(out_dict) + "\n")


def read_and_clear_histograms(dwell_time, num_histos, file_name):
    """
    Acquire histograms in read_and_clear mode.  During each histogram read,
    the ARM processor automatically delees the on-board histogram.
    We assume that only one MCA-1000 is attached, and that the MCA Data Server is running.

    :param dwell_time: Acquisition time in seconds
    :param num_histos: Number of histograms to acquire and write to file
    :param file_name: Write the histogram to this file as a on-line json file
    :return: None
    """
    mds_client, sn = bpi_communication.connect_one_device()

    start_mca_cmd = {"type": "mca_cmd", "name": "arm_ctrl",  "dir": "rmw", "sn": sn,
        "data": {"user":{"histogram_run": 1, "active_bank": 0, "read_clear": 1, "acq_type": 0,
                         "clear_statistics": 1, "clear_histogram": 1},
                 "fields": {"run_time_sample": 0}}}

    read_histogram_cmd = {"type": "mca_cmd", "name": "arm_histogram", "dir": "read", "sn": sn}
    read_status_cmd = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}

    # Delete a pre-existing file
    with open(file_name, 'w') as fout:
        pass

    # Start new sample histogram acquisition
    ret = mds_client.sar(start_mca_cmd)
    for n in range(num_histos):
        time.sleep(dwell_time)  # wait

        # Read count rate and histogram data
        ret = mds_client.sar(read_status_cmd)
        status_data = ret[sn]["fields"]

        ret = mds_client.sar(read_histogram_cmd)
        mca_data = ret[sn]["fields"]

        out_dict = {"status": status_data, "histogram": mca_data}
        with open(file_name, 'a') as fout:
            fout.write(json.dumps(out_dict) + "\n")


# acquire_histograms(dwell_time=1, num_histos=10, file_name="../data/histo_log.dat")

read_and_clear_histograms(dwell_time=10, num_histos=10, file_name="../data/histo_log.dat")

