#!/usr/bin/python
#
from __future__ import division
import time
import math
import json
import time
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com

version = 1.0


def acquire_one_histogram(par):
    """
    We assume that only one MCA-2000 is attached, and that the MCA Data Server is running.
    Note that 'sar' stands for send and receive.

    """
    # Unpack the input parameters for use below
    histo_4k = par["histo_4k"] & 1
    dwell_time = par["dwell_time"]
    file_name = par["file_name"]
    num_items = 2048*(1+histo_4k)  # 4096 vs 2048 MCA bins to read

    # Open communication with MCA Data Server
    # 127.0.0.1 is the same as localhost
    mds_client = com.zmq_device("tcp://127.0.0.1:5507", "client")  # Communicate with MDS

    # get serial number
    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret["sn_list"]
    for sn in sn_list:
        print("Unique serial number:", sn)
    sn = sn_list[0]  # We just serve one device here

    # Start new sample histogram acquisition
    cmd = {"type": "mca_cmd", "name": "arm_ctrl",  "dir": "rmw", "sn": sn,
           "data": {"fields": {"run_time_sample": dwell_time},
           "user":{"histogram_run": 1, "active_bank": 0, "histo_4k": histo_4k,
                   "clear_statistics": 1, "clear_histogram": 1}}}
    mds_client.sar(cmd)

    time.sleep(dwell_time+1)  # Wait 1 extra second to be sure the MCA has stopped.

    # The histogram acquisition has completed; now read the data.

    # Read histogram data
    cmd = {"type": "mca_cmd", "name": "arm_histogram", "dir": "read", "sn": sn, "num_items": num_items}
    ret_histogram = mds_client.sar(cmd)
    histogram = ret_histogram[sn]["registers"]

    # Read arm_status, includes count rate data
    cmd = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}
    ret_status = mds_client.sar(cmd)
    arm_status = ret_status[sn]

    # Read arm_ctrl, includes all instrument settings
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "read", "sn": sn}
    ret_ctrl = mds_client.sar(cmd)
    arm_ctrl = ret_ctrl[sn]

    # We have all the data; now save them to file.

    # Write a json file, with all data
    out_dict = {"sn":sn, "arm_ctrl": arm_ctrl, "arm_status": arm_status, "histogram": histogram}
    out_file = "{}.json".format(file_name)
    with open(out_file, 'w') as fout:
        fout.write(json.dumps(out_dict) + "\n")

    # Write histo as one column in a csv file
    out_file = "{}.dat".format(file_name)
    with open(out_file, 'w') as fout:
        out_str = "\n".join(["{:.0f}".format(h) for h in histogram])
        fout.write(out_str)


par = {
    "histo_4k": 0, # 0-> 2Kx32 histogram, 1-> 4Kx32 histogram
    "dwell_time": 10,  #  Data acquisition time in seconds
    "file_name": "../data/one_histo"  # Output file name without extension
}
acquire_one_histogram(par)

