import json
import matplotlib.pyplot as plt
import plot_lm_controls as pc
import math
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'import' directory in the parent directory also.
import bpi_plots


def get_listmode(file_name):
    times = []
    energies = []
    tars = []  # Time of arrivals
    number_of_events = 0
    with open(file_name, "r") as fin:
        for line in fin:
            dd = json.loads(line)
            num_events = dd["num_events"]
            #num_events = 500
            decimation = dd["decimation"]
            energies += dd["energies"][0:num_events]
            times += dd["times"][0:num_events]
            
            tt = dd["times"]
            dt = tt[num_events-1]-tt[0]
            if dt > 1e6:
                print("CR=", num_events/(dt))
            #print(num_events, dt)
    
    # histogram the energies    
    histo = [0]*4096
    for e in energies:
        histo[e] += 1
        
    # Time of arrival histogram
    tar_histo = [0]*1024
    tars = []
    for n in range(1, len(times)):
        dt = times[n] - times[n-1]
        if dt > 0:
            tars += [dt]  
    dt = 2**decimation / 24e6  # list mode time resolution
    for t in tars:
        idx = int(t/dt+0.5)
        if idx<len(tar_histo):
            tar_histo[idx] += 1
        
    return energies, times, histo, tar_histo, dt


energies, times, histo, tar_histo, dt = get_listmode("../data/lm2b_high_cr.dat")
x_histo = [x for x in range(4096)]
x_tar_histo = [x*dt*1e6 for x in range(len(tar_histo))]
x_events = [x for x in range(len(energies))]

bpi_plots.line_plot(x_events, energies, pc.energies_plot_ctrl)
bpi_plots.line_plot(x_events, times, pc.times_plot_ctrl)
bpi_plots.line_plot(x_histo, histo, pc.histo_plot_ctrl)
bpi_plots.line_plot(x_tar_histo, tar_histo, pc.tar_plot_ctrl)

plt.show()
