import json
import time
import zmq
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com


def log_status(par):
    """
    We assume that only one MCA is attached, and that the MCA Data Server is running.
    Continuously read the current status data from the ARM processor and append to log file.

    :param file_name: Write the status data to this file.
    :param dwell_time: Time between measurements, in seconds
    :param number_of_measurements: Number of measurements to record
    :return:
    """
    file_name = par["file_name"]
    dwell_time = par["dwell_time"]
    number_of_measurements = par["number_of_measurements"]

    mds_ip = "tcp://127.0.0.1:5507"
    mds_client = com.zmq_device(mds_ip, "client")

    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret['sn_list']
    for sn in sn_list:
        print("Serial number:", sn)
    sn = sn_list[0]  # We only use one attached device here

    with open(file_name, 'w') as fout:  # Delete old data
        pass
    cmd_read = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}
    for n in range(number_of_measurements):
        time.sleep(dwell_time)
        # Read arm_status
        ret = mds_client.sar(cmd_read)

        with open(file_name, 'a') as fout:
            fout.write(json.dumps(ret)+'\n')

par = {
    "file_name": "../data/status_log.dat",
    "dwell_time": 5,
    "number_of_measurements": 12
}
log_status(par)
